/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import javax.swing.JFrame;

public abstract class IOGame extends JFrame {
	private static final long serialVersionUID = -3110027951390203125L;
	protected Game game;
	
	/**
	 * Cree une vue, permettant de faire le lien entre le jeu et l'utilisateur.
	 * @param game Controleur du jeu
	 */
	protected IOGame(Game game) {
		this.game = game;
	}

	/**
	 * Demande au joueur de saisir son nom (sans possibilite de ne pas en ajouter).
	 * Retourne une InterruptedException si le joueur clique sur Annuler ou ferme la fenetre
	 * (et interrompt donc le jeu).
	 * @return Nom du joueur
	 * @throws InterruptedException
	 */
	abstract public String inFirstPlayerName() throws InterruptedException;
	
	/**
	 * Demande au joueur de saisir son nom (sans possibilite de ne pas en ajouter), avec la taille du
	 * plateau.
	 * Retourne une InterruptedException si le joueur clique sur Annuler ou ferme la fenetre
	 * (et interrompt donc le jeu).
	 * @return Nom du joueur
	 * @throws InterruptedException
	 */
	abstract public String inFirstPlayerNameWithFieldWidth() throws InterruptedException;
	
	/**
	 * Demande au joueur de saisir un nom (avec possibilite de ne pas en ajouter).
	 * @return Nom du joueur
	 * @throws InterruptedException
	 */
	abstract public String inPlayerName();
	
	/**
	 * Demande au joueur de proposer un mot.
	 * @return Mot propose
	 */	
	abstract public String inWord();
	
	/**
	 * Demande au joueur de choisir un type de jeu.
	 */
	abstract public void inGameMode();
	
	/**
	 * Demande au joueur client de saisir l'adresse du serveur hote. 
	 * @return Adresse du serveur hote
	 * @throws InterruptedException
	 */
	abstract public String inHostAddress() throws InterruptedException;
	
	/**
	 * Demande au joueur serveur d'interrompre la collecte des joueurs distants. Retourne
	 * une InterruptedException si le joueur decide d'attaquer un nouveau jeu sans lancer la partie. 
	 * @return Vrai, si le joueur interrompt la collecte des joueurs distants
	 * @throws InterruptedException
	 */
	abstract public boolean inNoNewRemotePlayers() throws InterruptedException;
	
	/**
	 * Demande au joueur si il veut rejouer.
	 */
	abstract public void inReplay();

	/**
	 * Affiche une erreur de nom de joueur (deja utilise, typiquement).
	 */
	abstract public void outErrorPlayerName();
	
	/**
	 * Affiche une erreur d'impossibilite de ne pas rentrer de nom de joueur
	 * (premier joueur).
	 */
	abstract public void outErrorPlayerRequired();
	
	/**
	 * Affiche une erreur de connection, avec le serveur.
	 */
	abstract public void outErrorConnection();
	
	/**
	 * Affiche une erreur concernant le mot saisi.  
	 * @param msg Expose du critere du mot non respecte
	 */
	abstract public void outErrorWord(String msg);

	/**
	 * Signale l'arrivee d'un joueur distant, au serveur.
	 * @param name Nom du joueur distant
	 */
	abstract public void outNewRemotePlayer(String name);
	
	/**
	 * Depart du programme, affiche la banniere du jeu.
	 */
	abstract public void outBanner();
	
	/**
	 * Signale que le dictionnaire se charge.
	 */
	abstract public void outBeginWaitLexicon();
	
	/**
	 * Signale que le dictionnaire est fini d'etre charge.
	 */
	abstract public void outEndWaitLexicon();
	
	/**
	 * Signale au joueur client qu'il doit attendre que le serveur lance la partie.
	 */
	abstract public void outBeginWaitServer();
	
	/**
	 * Signale au joueur que le serveur a lance la partie.
	 */
	abstract public void outEndWaitServer();
	
	/**
	 * Indique le joueur qui doit jouer (mode multi).
	 * @param name
	 */
	abstract public void outPlayerPlaying(String name);
	
	/**
	 * Indique que le mot propose est correct.
	 */
	abstract public void outCorrectWord();
	
	/**
	 * Met a jour le chrono (sert surtout pour la version GUI).
	 */
	abstract public void outUpdateTimer();
	
	/**
	 * Met a jour les scores (sert surtout pour la version GUI).
	 */
	abstract public void outUpdateScores();
	
	/**
	 * Affiche le plateau.
	 */
	abstract public void outBoard();
	
	/**
	 * Indique le temps de la partie est ecoule.
	 */
	abstract public void outElapsedTime();
	
	/**
	 * Affiche les resultats de la partie.
	 */
	abstract public void outResults();
	
	/**
	 * Fait du cafe.
	 */
	abstract public void outEE();

	/**
	 * Remet a zero l'interface (utilise surtout pour le GUI).
	 */
	abstract public void reset();
}