/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.util.Scanner;

public class IOGameText extends IOGame {
	private static final long serialVersionUID = 8479988412455521841L;
	private Scanner typing = new Scanner(System.in);

	/**
	 * Cree une vue texte.
	 * @param game Controleur du jeu
	 */
	public IOGameText(Game game) {
		super(game);
	}

	@Override
	public void inGameMode() {
		Game.GAME_MODE mode;
		System.out.print("\n (A)lone, (M)ultiplayers, (O)nline ? ");
		
		switch(typing.nextLine().toUpperCase().toCharArray()[0]) {
			// Jeu solo
			case 'A' :
				mode = Game.GAME_MODE.SOLO;
			break;

			// Jeu multijoueurs (sur la meme machine)
			case 'M' :
				mode = Game.GAME_MODE.MULTI;
			break;

			// Jeu en ligne
			case 'O' :
				System.out.print(" (H)ost, (J)oin ? ");
				
				switch(typing.nextLine().toUpperCase().toCharArray()[0]) {
				
					// Qui heberge
					case 'H' :
						mode = Game.GAME_MODE.ONLINE_HOST;
					break;

					// Qui rejoint
					case 'J' :
						mode = Game.GAME_MODE.ONLINE_CLIENT;
					break;

					// On recommence si ni l'un ni l'autre
					default :
						inGameMode();
						return;
				}
			break;

			// On recommence si ni l'un ni l'autre
			default :
				inGameMode();
				return;
		}

		game.setGameMode(mode);
	}

	@Override
	public void outBeginWaitLexicon() {
		System.out.println("\n\t((( Loading lexicon, please wait. )))\n");
	}	
	
	@Override
	public String inHostAddress() {
		System.out.print("\n Host IP : ");

		return typing.nextLine();
	}
	
	/**
	 * Affiche une boite en ASCII Art pour les titres principalement.
	 * @param msg Message a afficher
	 */
	private void outBox(String msg) {
		System.out.println("\n +------------------------------------------+");
		System.out.format(" | %-40s |\n", msg);
		System.out.println(" +------------------------------------------+\n");
	}

	@Override
	public String inPlayerName() {
		System.out.print(" Add a player (type a name, or just Enter) : ");

		return typing.nextLine();
	}
	
	@Override
	public String inFirstPlayerName() {
		System.out.print(" Add a player (type a name) : ");

		return typing.nextLine();
	}
	
	@Override
	public String inFirstPlayerNameWithFieldWidth() {
		try {
			System.out.print(" Field's width : ");
			game.getBoard().setSize(Integer.parseInt(typing.nextLine()));
			
			if(game.getBoard().getSize().width < 2)
				throw new NumberFormatException();

		} catch (NumberFormatException e) {
			System.out.println(" /!\\ Please, type a width greater than 2.\n");

			return inFirstPlayerNameWithFieldWidth();
		}

		return inFirstPlayerName();
	}

	@Override
	public String inWord() {
		System.out.print("\tPropose a word (or just Enter) : ");		
		String word = typing.nextLine();

		if(word.isEmpty() && (game.getGameMode() == Game.GAME_MODE.ONLINE_CLIENT || game.getGameMode() == Game.GAME_MODE.ONLINE_HOST))
			System.out.println("\n\tPlease, wait the others players or the timer end.");

		if(word.equals("42"))
			outEE();

		return word;
	}

	@Override
	public boolean inNoNewRemotePlayers() {
		System.out.println("\n Type Enter to stop the collecte.");
		typing.nextLine();

		return true;
	}
	
	@Override
	public void inReplay() {
		System.out.print("\n Replay (y/n) ? ");
		
		if(!typing.nextLine().equals("y"))
			System.exit(0);
	}

	@Override
	public void outErrorPlayerRequired() {
		System.out.println("\t/!\\ Please type a name.\n");
	}
	
	@Override
	public void outErrorPlayerName() {
		System.out.println("\t/!\\ This name is already used.\n");
	}
	
	@Override
	public void outErrorConnection() {
		System.out.println("\t/!\\ Unable to connect server.\n");		
	}
	
	@Override
	public void outNewRemotePlayer(String name) {
		System.out.println(" - " + name + " has joined the game.");
	}
	
	@Override
	public void outErrorWord(String msg) {
		System.out.println("\t/!\\ " + msg + "\n");
	}

	@Override
	public void outBanner() {
		System.out.println("  _____   _____   _____   _____   _       _____");
		System.out.println(" |  _  \\ /  _  \\ /  ___| /  ___| | |     | ____|");
		System.out.println(" | |_| | | | | | | |     | |     | |     | |__");   
		System.out.println(" |  _  | | | | | | |  _  | |  _  | |     |  __|"); 
		System.out.println(" | |_| | | |_| | | |_| | | |_| | | |___  | |___");
		System.out.println(" |_____/ \\_____/ \\_____/ \\_____/ |_____| |_____|");
		System.out.println("\t\t ESIAL/TOP Project'11");
	}
	
	@Override
	public void outElapsedTime() {
		outBox("Timed out !");
	}

	@Override
	public void outEndWaitLexicon() {}

	@Override
	public void outPlayerPlaying(String name) {
		outBox(name + " plays");
	}

	@Override
	public void outCorrectWord() {
		System.out.println();
	}

	@Override
	public void outResults() {
		outBox("Words found");
		
		for(Player p : game.getPlayers())
			if(!(p instanceof PlayerBot))
				System.out.println("\t" + p + " : " + p.getWords().keySet());

		outBox("Original words found");

		for(Player p : game.getPlayers())
			if(!(p instanceof PlayerBot))
				System.out.println("\t" + p + " : " + p.getOriginalWords().keySet());
		
		outBox("Words not found");
		
		for(Player p : game.getPlayers())
			if(p instanceof PlayerBot)
				System.out.println("\t" + p.getOriginalWords().keySet());

		outBox("Scores");

		for(Player p : game.getPlayers())
			if(!(p instanceof PlayerBot))
				System.out.println("\t" + p + " : " + p.getScore());
	}
	
	@Override
	public void outBoard() {
		outBox("GAME BOARD");

		System.out.print(game.getBoard());
	}

	@Override
	public void outUpdateTimer() {}
	
	@Override
	public void reset() {}

	@Override
	public void outBeginWaitServer() {
		System.out.println("\n You're registred. Please wait server...");
	}

	@Override
	public void outEndWaitServer() {}

	@Override
	public void outUpdateScores() {}
	
	@Override
	public void outEE() {
		System.out.println("\t         _____");                     
		System.out.println("\t        /    /          .-''-.");     
		System.out.println("\t       /    /         .' .-.  )  ");  
		System.out.println("\t      /    /         / .'  / /     ");
		System.out.println("\t     /    /         (_/   / /      ");
		System.out.println("\t    /    /  __           / /       ");
		System.out.println("\t   /    /  |  |         / /        ");
		System.out.println("\t  /    '   |  |        . '         ");
		System.out.println("\t /    '----|  |---.   / /    _.-') ");
		System.out.println("\t/          |  |   | .' '  _.'.-''  ");
		System.out.println("\t'----------|  |---'/  /.-'_.'      ");
		System.out.println("\t           |  |   /    _.'         ");
		System.out.println("\t          /____\\ ( _.-'            \n");
	}
}