/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Scanner;
import java.util.zip.GZIPInputStream;

import org.ardverk.collection.*;

public class LexiconTrie {
	PatriciaTrie<String, Object> lexicon = new PatriciaTrie<String, Object>(StringKeyAnalyzer.INSTANCE);

	/**
	 * Cree un dictionnaire vide.
	 */
	public LexiconTrie() { }

	/**
	 * Cree un dictionnaire remplie des mots indique dans le fichier donne.
	 * @param file Fichier compresse des mots a inserer dans le dictionnaire
	 * (un mot par ligne)
	 * @throws IOException
	 */
	public LexiconTrie(String file) throws IOException {
		read(file);
	}
	
	/**
	 * Ajoute les mots d'un fichier dans le dictionnaire. 
	 * @param baseFileName Fichier de mots compresse (un mot par ligne)
	 * @throws IOException
	 */
	public void read(String baseFileName) throws IOException {
		
		// Decompresse le fichier
		FileInputStream baseFileGZ = new FileInputStream(baseFileName);
		GZIPInputStream baseFile = new GZIPInputStream(baseFileGZ);
		Scanner base = new Scanner(baseFile);

		// Ajoute les mots
		while(base.hasNextLine())
			add(base.nextLine().toUpperCase());
	}

	/**
	 * Ajouter un mot au dictionnaire.
	 * @param word Mot a ajouter
	 */
	public void add(String word) {
		lexicon.put(word.toUpperCase(), null);
	}

	/**
	 * Supprimer un mot du dictionnaire.
	 * @param word Mot a supprimer
	 */
	public void remove(String word) {
		lexicon.remove(word.toUpperCase());
	}

	/**
	 * Indique si un mot est present dans le dictionnaire.
	 * @param word Mot a chercher
	 * @return Vrai si le mot est present dans le dictionnaire
	 */
	public boolean contains(String word) {
		return lexicon.containsKey(word.toUpperCase());
	}

	/**
	 * Indique si le mot donne est le prefixe d'au moins un mot dans le dictionnaire.
	 * @param prefix Mot a tester
	 * @return Vrai si le mot donne est le prefixe d'au moins un mot dans le dictionnaire
	 */
	public boolean mayContain(String prefix) {
		return lexicon.select(prefix).getKey().startsWith(prefix);
	}

	/**
	 * Retourne le nombre de mots dans le dictionnaire.
	 * @return Nombre de mots dans le dictionnaire
	 */
	public int size() {
		return lexicon.size();
	}
	
	/**
	 * Indique si le dictionnaire est vide.
	 * @return Vrai si le dictionnaire est vide
	 */
	public boolean isEmpty() {
		return lexicon.isEmpty();
	}
}