/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.io.*;
import java.net.Socket;

public abstract class Network {
	protected Game game;
	protected Socket connection;
	static final protected int PORT_MAIN = 42000, PORT_SYNCHRO = 42001;
	static private String host;
	
	/**
	 * Retourne l'IP du serveur
	 */
	public static String getHost() {
		return host;
	}

	/**
	 * Desactive un joueur distant injoignable, ou coupe la partie si c'est le serveur qui
	 * ne repond plus.  
	 * @param e IOException capturee lors du probleme de reseau rencontre (relancee si c'est le serveur
	 * qui pose probleme, puisque la partie ne peut pas continuer)
	 * @throws IOException
	 */
	private void reset(IOException e) throws IOException {
		String address = connection.getInetAddress().getHostAddress();

		// C'est le serveur qui injoignable (cas d'un client), le jeu est interrompu
		if(address.equals(Network.host))
			throw e;
		
		// Il s'agit d'un joueur distant (cas d'un serveur), on le desactive simplement
		else
			game.disableRemotePlayer(game.getRemotePlayerByAddress(address), this);
	}

	/**
	 * Envoi de donnees a un joueur/serveur distant.
	 * @doc http://www.jguru.com/faq/view.jsp?EID=34789
	 * @param msg Objet a envoyer
	 * @throws IOException
	 */
	public void send(Serializable msg) throws IOException {
		try {
	        ObjectOutputStream out = new ObjectOutputStream(connection.getOutputStream());		
			out.writeObject(msg);
			out.flush();

		// Le correspondant est injoignable
		} catch(IOException e) {
			reset(e);
		}
	}
	
	/**
	 * Reception de donnees d'un joueur/serveur distant.
	 * @return Objet recu
	 * @throws IOException
	 */
	public Object receipt() throws IOException {
		Object o = null;

		try {
			ObjectInputStream in = new ObjectInputStream(connection.getInputStream());
			o = in.readObject();

		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		return o;
	}
	
	/**
	 * Fermeture du socket vers le joueur/serveur 
	 */
	public void disconnect() {
		try {
			connection.close();
		} catch (IOException e) {}
	}
	
	/**
	 * Permet de definir l'IP du serveur hote
	 * @param host IP du serveur hote
	 */
	static public void setHost(String host) {
		Network.host = host;
	}
}