/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import org.ardverk.collection.PatriciaTrie;

public class PlayerBot extends Player {
	public final static String DEFAULT_NAME = "SpokeBot";
	
	/**
	 * Cree un joueur robot, avec le nom par defaut.
	 * @param game Controleur du jeu
	 */
	public PlayerBot(Game game) {
		super(DEFAULT_NAME, game);
	}

	/**
	 * Verifie le mot, en se contentant de verifier sa taille.
	 */
	@Override
	public void checkWord(String word) throws WordException {
		if(allWordsFound.containsKey(word))
			throw new WordException(null);
		
		game.minCheckWord(word);
	}

	/**
	 * Ajoute le mot a la liste des mots originaux, si aucun autre joueur ne l'a trouve
	 * (le robot est le dernier a jouer). Il s'agit donc de la liste des mots qui n'ont pas
	 * ete trouve sur le plateau.
	 */
	@Override
	public void addWord(String word) {
		boolean alreadyFound = false;

		for(Player p : game.getPlayers())
			if(p instanceof PlayerHuman && p.getWords().containsKey(word))
				alreadyFound = true;

		if(!alreadyFound)
			add2OriginalWords(word);
	}
	
	/**
	 * Le robot joue, en trouvant l'integralite des mots trouvables sur le plateau.
	 */
	@Override
	public void play() {
		PatriciaTrie<String, Object> solutions;

		// Integralite des solutions
		solutions = game.getBoard().allReadableInBoard(
			game.getLexicon()
		);

		// Ajout du mot a la liste des originaux (peut importe l'autre liste)
		for(String word : solutions.keySet()) {
			try {
				checkWord(word);
				addWord(word);
			} catch(WordException e) { }
		}
		
		// Le robot a fini de jouer
		finish();
	}

	/**
	 * Change la liste des mots originaux du robot (utile pour le client).
	 * @param originalWordsFound Liste des mots qui n'ont pas ete trouves sur le plateau, par les joueurs 
	 */
	public void setWords(PatriciaTrie<String, Object> originalWordsFound) {
		this.originalWordsFound = originalWordsFound;
	}
}