/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import org.ardverk.collection.PatriciaTrie;

public abstract class PlayerClient extends PlayerNetwork {

	/**
	 * Ajoute un joueur local/distant sur une machine cliente (depuis un heritage) 
	 * @param name Nom du joueur
	 * @param game Controleur du jeu
	 * @param address Adresse IP du joueur
	 */
	public PlayerClient(String name, Game game, String address) {
		super(name, game, address);
	}

	/**
	 * Un joueur sur une partie cliente ne gere pas ses propres mots, le serveur fait tout.
	 * Fonction vide. 
	 */
	@Override
	public void addWord(String word) {}

	/**
	 * Retourne un score recu au prealable par le serveur, qui les gere
	 */
	@Override
	public int getScore() {
		return score;
	}

	/**
	 * Permet de definir le score du joueur, recu par le serveur
	 * @param score Score du joueur
	 */
	public void setScore(int score) {
		this.score = score;
	}

	/**
	 * Permet de definir les listes de mots trouves pour le joueur, recues par le serveur
	 * @param allWordsFound Liste exaustives des mots trouves
	 * @param originalWordsFound Liste des mots originaux
	 */
	public void setWords(PatriciaTrie<String, Object> allWordsFound, PatriciaTrie<String, Object> originalWordsFound) {
		this.allWordsFound = allWordsFound;
		this.originalWordsFound = originalWordsFound;
	}
}