/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.io.IOException;

public class PlayerHuman extends Player {
	protected int score;

	/**
	 * Cree un joueur humain (!bot)
	 * @param name Nom du joueur
	 * @param game Controleur du jeu
	 */
	public PlayerHuman(String name, Game game) {
		super(name, game);
		score = 0;
	}
	
	/**
	 * Verifie l'integralite des conditions pour que le mot soit accepte.
	 */
	@Override
	public void checkWord(String word) throws WordException, IOException {
		if(allWordsFound.containsKey(word))
			throw new WordException("You've already found this word.");

		game.fullCheckWord(word);
	}

	/**
	 * Invite l'utilisateur a jouer directement en proposant des mots.
	 * Renvoi une InterruptedException si la partie est brutalement arretee.
	 */
	@Override
	public void play() throws IOException, InterruptedException {
		String word;

		word = game.getIO().inWord().toUpperCase();

		// Permet d'interrompre le jeu
		if(game.isInterrupted())
			throw new InterruptedException();
		
		// Permet de stopper le jeu du joueur
		while(!word.isEmpty() && game.isStarted()) {
			try {
				// Permet d'interrompre le jeu
				if(game.isInterrupted())
					throw new InterruptedException();
				
				// Verifie le mot, et l'ajoute si aucune exception n'a fait sauter dans le catch
				checkWord(word);
				addWord(word);

				game.getIO().outUpdateScores();
				game.getIO().outCorrectWord();

			// Le mot ne correspond pas aux criteres
			} catch(WordException e) {
				game.getIO().outErrorWord(e.getMessage());
			}

			word = game.getIO().inWord().toUpperCase();
			
			// Permet d'interrompre le jeu
			if(game.isInterrupted())
				throw new InterruptedException();
		}

		// Le joueur a fini de jouer
		finish();
	}
}