/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.net.InetAddress;

abstract public class PlayerNetwork extends PlayerHuman {
	private String address;
	public final static String IDENTITY_SEP = "/";

	/**
	 * Cree un joueur reseau (distant, ou local sur une machine cliente).
	 * @param name Nom du joueur
	 * @param game Controleur du jeu
	 * @param address IP du joueur
	 */
	public PlayerNetwork(String name, Game game, String address) {
		super(name, game);
		this.address = address;
	}

	/**
	 * Retourne l'identite du joueur, avec le DEFAULT_HOST d'un joueur local si non surchargée.
	 * @return Identite du joueur, avec le DEFAULT_HOST d'un joueur local si non surchargée.
	 */
	public String getLocalIdentity() {
		return PlayerClientLocal.DEFAULT_HOST + IDENTITY_SEP + name;
	}

	/**
	 * Returne l'identite en version distante (avec l'IP) du joueur.
	 * @return Identite en version distante (avec l'IP) du joueur
	 */
	public String getRemoteIdentity() {
		return address + IDENTITY_SEP + name;
	}
	
	/**
	 * Affichage du nom du joueur, avec son IP.
	 */
	public String toString() {
		return name + " (" + address + ")";
	}
	
	/**
	 * Retourne l'adresse IP du joueur.
	 * @return Adresse IP du joueur
	 */
	public String getAddress() {
		return address;
	}
	
	/**
	 * Compare le joueur, avec une adresse et determine si c'est la sienne ou non.
	 * @param iAddress Adresse a comparer
	 * @return Vrai si il s'agit de l'adresse du joueur
	 */
	public boolean equals(InetAddress iAddress) {
		return iAddress.getHostAddress().equals(address);
	}
}