/*
 * Boggle - TOP PROJET ESIAL 2011
 * Copyright (C) 2011 Julien VAUBOURG / Igor MARTY
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package boggle;

import java.io.IOException;

public class PlayerRemote extends PlayerNetwork {
	boolean disabled;
	
	/**
	 * Cree un joueur distant, pour la machine serveur.
	 * @param name Nom du joueur
	 * @param game Controleur du jeu
	 * @param address Adresse IP du joueur
	 */
	public PlayerRemote(String name, Game game, String address) {
		super(name, game, address);
	}
	
	/**
	 * Desactive un joueur (theoriquement, apres avoir tue ses threads reseaux).
	 */
	public void disable() {
		disabled = true;
		finish();
	}
	
	/**
	 * Retourne l'etat du joueur.
	 * @return Vrai si le joueur a ete desactive
	 */
	public boolean isDisabled() {
		return disabled;
	}

	/**
	 * Permet au joueur distant de jouer, avec le mot recu de sa part par le thread serveur.
	 * Renvoi une WordException comme la vraie fonction play, si le mot n'est pas correct.
	 * @param word Mot a jouer
	 * @throws WordException
	 * @throws IOException
	 */
	public void play(String word) throws WordException, IOException {
		checkWord(word);
		addWord(word);
	}
}