package tests;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import boggle.Dice;
import boggle.Game;
import boggle.LexiconTrie;
import boggle.Player;
import boggle.PlayerHuman;
import boggle.WordException;

import junit.framework.TestCase;

public class GameTest extends TestCase {

	/**
	 * Teste si les mots sont bien verifies.
	 */
	public void testCheckWord() {
		LexiconTrie l = new LexiconTrie();
		Game g = new Game(l);
		Player p = new PlayerHuman("Assange", null);
		g.addPlayer(p);
		
		l.add("ADAM");
		l.add("DOUGLAS");
		l.add("INTERGALACTIC");

		// Teste d'un mot trop petit
		try {
			g.fullCheckWord("42");
			fail();
		} catch (WordException e) {
			assertTrue(e.getMessage().contains("shorter"));
		}

		// Teste d'un mot qui est sur le plateau, a la bonne taille mais n'est pas dans le dico
		try {
			g.fullCheckWord("HITCHHIKERS");
			fail();
		} catch (WordException e) {
			assertTrue(e.getMessage().contains("in the lexicon"));
		}

		// Teste d'un mot qui est sur le plateau et dans le dico, a la bonne taille 
		try {
			g.fullCheckWord("DOUGLAS");
			assertTrue(true);
		} catch (WordException e) {
			fail();
		}
		
		p.add2Words("DOUGLAS");
		
		// Teste d'un mot qui a deja ete ajoute
		try {
			g.fullCheckWord("DOUGLAS");
			fail();
		} catch (WordException e) {
			assertTrue(e.getMessage().contains("already"));
		}
	}

	/**
	 * Teste du calcul des scores.
	 */
	public void testWordScore() {
		Game g = new Game(new LexiconTrie());

		assertEquals(g.wordScore("galaxy"), 3);
		assertEquals(g.wordScore("TCP"), 1);
	}

	/**
	 *  Teste du setField.
	 */
	public void testSetField() {
		try {
			String fileName = "dice-definition.txt";
			String[] sed = { "/bin/sh", "-c", "sed -n '/^[^;]/p' " + fileName + " | sed -n 4p" };
			Process p = Runtime.getRuntime().exec(sed);
			BufferedReader stdin = new BufferedReader(new InputStreamReader(p.getInputStream()));
			char firstSideOf4th = stdin.readLine().charAt(0);
			
			Game game = new Game();
			game.getBoard().setField(fileName);

			Dice[][] d = game.getBoard().getField();
			assertEquals(d[1][1].getValue(), firstSideOf4th);

		} catch (FileNotFoundException e) {
			System.err.println("Dice definition file not found for test !");

		} catch (IOException t) {
			System.err.println("OMG, I'm afraid you're on Windows :/");
		}
	}

	/**
	 *  Teste de l'ajout de joueur.
	 */
	public void testAddPlayer() {
		try {
			Game g;
			String[] stdin = { "Assange", "Assange", "Julian" };

			g = new Game(new LexiconTrie());
			g.setIO(new IOGameTest(stdin));

			g.addPlayer();

			// Teste d'un nom sans probleme
			assertEquals(g.getPlayers().get(0).getName(), "Assange");
	
			// Teste d'un nom deja rentre 
			g.addPlayer();
			assertEquals(g.getPlayers().get(1).getName(), "Julian");
	
			String[] stdin2 = { "", "Assange" };
			g = new Game();
			g.setIO(new IOGameTest(stdin2));
	
			// Teste d'un nom vide
			g.addPlayer();
			assertEquals(g.getPlayers().get(0).getName(), "Assange");

		} catch (InterruptedException e) {}
	}
}